from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User, Permission
from django.template.defaultfilters import slugify
from django.core.urlresolvers import reverse
from pgauth.base import LocalRoleMixin, Role
from pgauth.models import UserGroup
from pgauth.settings import VIEWER_ROLE_ID
from djinn_contenttypes.models.sharing import SharingMixin
from djinn_contenttypes.models.relatable import RelatableMixin
from djinn_workflow.utils import get_state


class BaseContent(models.Model, LocalRoleMixin, SharingMixin, RelatableMixin):

    """ All Djinn content extends this base class. Or should... """

    title = models.CharField(_('Title'), max_length=200)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)
    creator = models.ForeignKey(User, related_name='%(class)s_creator')
    changed_by = models.ForeignKey(User, related_name='%(class)s_changed_by')
    removed_creator_name = models.CharField(_('Creator naam'), max_length=100,
                                            blank=True, null=True)
    userkeywords = models.CharField(_('Keywords'), max_length=500,
                                    null=True, blank=True)
    parentusergroup = models.ForeignKey(
        UserGroup,
        related_name='%(class)s_parentusergroup',
        null=True, blank=True, on_delete=models.SET_NULL)
    is_tmp = models.BooleanField(_('Temporary'), default=False)

    # Create a temporary object during add?
    #
    create_tmp_object = False

    def get_cache_key(self):

        return "%s_%s_%s" % (self.app_label, self.ct_name, self.id)

    def save(self, *args, **kwargs):

        if self.userkeywords:
            self.userkeywords = self.userkeywords.replace("'", '')
            kw_list = self.userkeywords.split(',')[:10]
            self.userkeywords = ",".join([kw.strip() for kw in kw_list])
        super(BaseContent, self).save(*args, **kwargs)

    def __unicode__(self):

        return self.title

    @property
    def slug(self):

        return slugify(self.title)

    @property
    def ct_class(self):

        """ Use this method at your own peril... it's expensive"""

        return ContentType.objects.get_for_model(self.__class__)

    @property
    def ct_name(self):
        return self.__class__.__name__.lower()

    @property
    def app_label(self):

        try:
            return self._meta.app_label
        except:
            return self.__module__.split(".")[0]

    @property
    def ct_label(self):

        """ Display name """

        return _(self.__class__.__name__)

    @property
    def in_closed_group(self):

        return self.parentusergroup and self.parentusergroup.profile and \
            self.parentusergroup.profile.is_closed

    @property
    def is_public(self):

        """ We're public iff:
            * not in a closed group
            * we have an id
            * we're not temporary
            * the state is not private
            * we're not deleted...
        """

        if not self.id:
            return False
        elif self.is_tmp:
            return False
        elif self.in_closed_group:
            return False
        elif get_state(self).name == "private":
            return False
        elif self.is_deleted:
            return False
        else:
            return True

    @property
    def is_deleted(self):

        try:
            self.__class__.objects.get(pk=self.id)
            _deleted = False
        except self.__class__.DoesNotExist:
            _deleted = True
        return self.created and _deleted

    @property
    def permission_authority(self):

        """ Return the permission authority for this object. This is
        usually the object itself, but in some cases, for example
        child objects like user profile phonenumbers, the check may be
        deferred to the parent object """

        return self

    @property
    def keywordslist(self):
        if self.userkeywords:
            return self.userkeywords.split(',')
        return []

    def pre_delete(self):

        """ May raise DeleteError """

        pass

    def delete(self):

        for lrole in self.get_local_roles():
            lrole.delete()

        return super(BaseContent, self).delete()

    def get_absolute_url(self):

        return reverse('%s_view_%s' % (self.app_label, self.ct_name),
                       kwargs={"slug": self.slug, "pk": str(self.id)})

    def get_local_roles(self, **kwargs):

        """ local roles are assigned as follows:
            1. if the content is public, the viewer role is given to all
            2. if the content is private, no one is allowed
            3. if the content is not public, but also not private,
               group members are given access.
        """

        roles = super(BaseContent, self).get_local_roles(**kwargs)

        if kwargs.get("as_role", False):

            if self.is_public:

                viewer = Role.objects.filter(
                    name=VIEWER_ROLE_ID).select_related()
                roles = roles | viewer

            elif get_state(self).name == "private" or not self.is_published:

                pass

            elif self.in_closed_group and kwargs.get("user", None):

                if kwargs['user'] in self.parentusergroup.members.all():

                    viewer = Role.objects.filter(
                        name=VIEWER_ROLE_ID).select_related()

                    roles = roles | viewer

        return roles

    @property
    def viewers(self):

        """ Return a list of all unique users and groups that can
        'view' this content."""

        _viewers = set()

        view = Permission.objects.get(codename="view")

        if self.is_public:
            _viewers.add("group_users")
        elif self.parentusergroup and self.is_published and get_state(
                self).name != "private":

            _viewers.add("group_%d" % self.parentusergroup.id)

        for lrole in self.get_local_roles():
            if view in lrole.role.permissions.all():

                try:
                    viewuser = lrole.user
                except User.DoesNotExist:
                    viewuser = None

                if viewuser:
                    _viewers.add("user_%s" % viewuser.username)
                else:
                    try:
                        viewusergroup = lrole.usergroup
                    except UserGroup.DoesNotExist:
                        viewusergroup = None

                    if viewusergroup:
                        _viewers.add("group_%d" % viewusergroup.id)

        return list(_viewers)

    @property
    def acquire_global_roles(self):

        """ Do we need to acquire global roles? Only if published... """

        if not self.is_public:
            return False
        else:
            return True

    class Meta:
        abstract = True


class FKContentMixin(object):

    """ Marker class for content that has a foreign key to some parent """



from datetime import datetime
from django.db import models
from django.utils.translation import ugettext_lazy as _
from djinn_contenttypes.models.base import BaseContent


class PublishableMixin(object):

    @property
    def is_published(self):

        """The content is considered published if either dates are not set, or
        publish_from is earlier than now, and publish_to is later than
        now.  Note that this is not the same as 'is_public', since
        that also depends on visibility!

        """

        now = datetime.now()

        return ((not self.publish_from or self.publish_from < now)
                and
                (not self.publish_to or self.publish_to > now))

    @property
    def is_scheduled(self):

        """Determine whether one of date_from or date_to are set.  If so, we
        consider publishing to be 'scheduled', not considering the
        actual dates.

        """

        return self.publish_from or self.publish_to

    @property
    def is_public(self):

        """ Are we public? This is true iff:
          - not initial still
          - state is public
          - publish_from is empty or earlier than now
          - publish_to is empty or later than now
        """

        return (
            super(PublishableMixin, self).is_public
            and
            not self.is_tmp
            and
            self.is_published)


class PublishableContent(PublishableMixin, BaseContent):

    """ Anything that is publishable should extend this class """

    publish_from = models.DateTimeField(_('Publish from'), null=True,
                                        db_index=True,
                                        blank=True, default=None)
    publish_to = models.DateTimeField(_('Publish to'), null=True,
                                      db_index=True,
                                      blank=True, default=None)
    publish_notified = models.BooleanField(
        _('Event sent'), default=False,
        help_text=_("Publish event is sent"))
    unpublish_notified = models.BooleanField(
        _('Event sent'), default=False,
        help_text=_("Un-publish event is sent"))
    remove_after_publish_to = models.BooleanField(
        _('Remove the content ater publication to has past'), default=False)

    @property
    def publishing_date(self):

        """ If publish_from is set, use that, otherwise use created """

        return self.publish_from or self.created

    class Meta:
        abstract = True



import requests
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db.models.base import ModelBase
from django.db.models import Model, get_model
from django.core import exceptions
from djinn_core.utils import implements
from djinn_contenttypes.registry import CTRegistry


def has_permission(perm, user, obj):

    """ Check whether the user has the permisson on the object (or the
    objects' permission_authority)."""

    authority = getattr(obj, "permission_authority", obj)

    if not authority:
        return True

    return user.has_perm(perm, obj=authority)


def get_model_name(model):

    "Returns the Python model class for this type of content."

    if isinstance(model, Model) or isinstance(model, ModelBase):
        return model._meta.object_name.lower()
    return str(model)


def get_contenttype_by_ct_name(ct_name, app_label=None):
    if app_label is None:
        app_label = CTRegistry.get(ct_name)['app']

    return ContentType.objects.get(app_label=app_label,
                                   model=ct_name)


def get_contenttype_by_ctype(ctype, app_label=None):
    if app_label is None:
        app_label = CTRegistry.get(get_model_name(ctype))['app']

    return ContentType.objects.get(app_label=app_label,
                                   model=get_model_name(ctype))


def get_object_by_ctype(ctype, _id, app_label=None):

    content_type = get_contenttype_by_ctype(ctype, app_label=app_label)

    return content_type.get_object_for_this_type(id=_id)


def get_object_by_ctype_id(ctype_id, _id, app_label=None):

    ctype = CTRegistry.get(ctype_id)['class']

    return get_object_by_ctype(ctype, _id, app_label)


def json_serializer(obj):

    if implements(obj, Model):

        obj_data = {}

        for field in obj._meta.fields:
            try:
                obj_data[field.name] = str(field.value_from_object(obj))
            except:
                pass

        return str(obj_data)

    return "NOT SERIALIZABLE"


def get_comment_model():

    if settings.DJINN_COMMENT_MODEL:

        try:
            parts = settings.DJINN_COMMENT_MODEL.split('.')

            model = get_model(parts[0], parts[-1])
        except:
            raise exceptions.ImproperlyConfigured('Erroneous comment model')

        return model


def check_get_url(url, cookies=None):

    """ return http status for fetching this url """

    return requests.get(url, cookies=cookies,
                        allow_redirects=False).status_code



from datetime import datetime
from django.core.management.base import BaseCommand
from django.db.models import Q
from djinn_contenttypes.models.publishable import PublishableContent
from djinn_contenttypes.registry import CTRegistry
from django.utils import translation


class Command(BaseCommand):

    help = """Publish contenttypes if need be"""

    def handle(self, *args, **options):

        """Check contentitems that are beyond the publish_to date.  We just
        call 'save' on the instance, and let the signal handlers take
        care of the rest.

        """

        now = datetime.now()
        translation.activate("nl_NL")

        for ctype in CTRegistry.list_types():

            model = CTRegistry.get_attr(ctype, "class")

            if issubclass(model, PublishableContent):

                for instance in model.objects.filter(
                        unpublish_notified=False,
                        publish_to__isnull=False).filter(
                            Q(publish_to__lt=now) | Q(publish_from__gt=now)):

                        # model.objects.filter(
                        # unpublish_notified=False,
                        # publish_to__isnull=False, publish_to__lt=now):

                    # be sure this unpublish is not handled again
                    instance.unpublish_notified = True

                    # prepare for a following publication of this instance
                    # print "un-publishing %s-%d" % (instance.ct_name, instance.id)
                    instance.publish_notified = False
                    instance.save()

                # clean up tenacious content...
                #
                for instance in model.objects.filter(
                        remove_after_publish_to=True,
                        publish_to__isnull=False, publish_to__lt=now):

                    instance.delete()



from django.test.testcases import TestCase
from djinn_contenttypes.models.relatable import RelatableMixin
from django.contrib.contenttypes.models import ContentType


class FakeContent(RelatableMixin):

    def __init__(self, _id):

        self.ct_class = ContentType.objects.get_for_model(ContentType)
        self.id = _id


RELATION = "SOME_RELATION"


class RelatableTest(TestCase):

    def setUp(self):

        self.source = FakeContent(1)
        self.target = FakeContent(2)

    def test_has_relation(self):

        self.assertFalse(self.source.has_relation(RELATION))
        self.assertFalse(self.target.has_relation(RELATION))

        self.source.add_relation(RELATION, self.target)

        # Ok, source should have a relation right now, target shouldn't...
        #
        self.assertTrue(self.source.has_relation(RELATION))
        self.assertFalse(self.target.has_relation(RELATION))

        # except when we ask for the inverse relation.
        #
        self.assertFalse(self.source.has_relation(RELATION, inverse=True))
        self.assertTrue(self.target.has_relation(RELATION, inverse=True))

        # Specifying the exact target should yield the same as without
        # target
        #
        self.assertTrue(self.source.has_relation(RELATION, self.target))
        self.assertFalse(self.source.has_relation(RELATION, self.source))

        # The target should have the inverse relation too!
        #
        self.assertTrue(self.target.has_relation(RELATION, inverse=True))
        self.assertTrue(self.target.has_relation(RELATION, inverse=True,
                                                 target=self.source))



from django.test.testcases import TestCase
from django.contrib.auth.models import User
from pgcontent.models import Article
from djinn_core.utils import object_to_urn, urn_to_object


class UtilsTest(TestCase):

    def setUp(self):

        super(UtilsTest, self).setUp()
        self.user = User.objects.create(username="bobdobalina",
                                        is_superuser=True)
        self.obj = Article.objects.create(title="Article 1",
                                          changed_by=self.user,
                                          creator=self.user)

    def test_object_to_urn(self):

        "urn:pu.in:%(object_app)s:%(object_ctype)s:%(object_id)s"

        self.assertEquals("urn:pu.in:pgcontent:article:%s" % self.obj.id,
                          object_to_urn(self.obj))

    def test_urn_to_object(self):

        self.assertEquals(
            self.obj,
            urn_to_object("urn:pu.in:pgcontent:article:%s" % self.obj.id))



import logging
from django.db import models


LOG = logging.getLogger("djinn_contenttypes")


class RelatableMixin(object):

    """ Mixin class that enables relations."""

    @property
    def relation_model(self):

        """ Dynamically fetch model, so as to remove module dependency """

        return models.get_model("pgcontent", "SimpleRelation")

    def get_related(self, relation_type=None, inverse=False):

        """ Return all related content. This is a costly operation, so
        use it wisely..."""

        if relation_type:
            relations = self.get_relations(relation_type_list=[relation_type],
                                           inverse=inverse)
        else:
            relations = self.get_relations()

        related = []

        for rel in relations:
            try:
                if inverse:
                    relation_source = rel.get_src()
                    relation_source._meta  # force a check
                    related.append(relation_source)
                else:
                    relation_target = rel.get_tgt()
                    relation_target._meta
                    related.append(relation_target)
            except:
                LOG.warn("Cleaning up broken relation %s", rel)
                rel.delete()

        return related

    def get_relations(self, relation_type_list=None, target_type=None,
                      inverse=False):

        """
        Get all relations for the given object. If relation_type
        is set, return only these relations. If inverse is true,
        return inverse relations.

        Returns a Filter result.
        """

        if not relation_type_list:
            relation_type_list = []

        if not inverse:
            _filter = {'src_content_type': self.ct_class,
                       'src_object_id': self.id}
        else:
            _filter = {'tgt_content_type': self.ct_class,
                       'tgt_object_id': self.id}

        if len(relation_type_list):
            _filter['relation_type__in'] = relation_type_list

        if target_type:
            _filter['tgt_content_type'] = target_type

        return self.relation_model.objects.filter(**_filter)

    def add_relation(self, relation_type, target, unique=True):

        """ Add relation with given type with target as receiving end.
        If unique is true, don't add if it's already there..."""

        if unique and self.has_relation(relation_type, target):
            return None

        return self.relation_model.objects.create(
            src_content_type=self.ct_class,
            src_object_id=self.id,
            relation_type=relation_type,
            tgt_content_type=target.ct_class,
            tgt_object_id=target.id)

    def has_relation(self, relation_type, target=None, inverse=False):

        """Do we have a relation? With target? If inverse is set, check
        that...

        """

        _filter = {'relation_type': relation_type}

        if inverse:
            _filter.update(
                {'tgt_content_type': self.ct_class,
                 'tgt_object_id': self.id})
        else:
            _filter.update(
                {'src_content_type': self.ct_class,
                 'src_object_id': self.id})

        if target:
            if inverse:
                _filter.update(
                    {'src_content_type': target.ct_class,
                     'src_object_id': target.id})
            else:
                _filter.update(
                    {'tgt_content_type': target.ct_class,
                     'tgt_object_id': target.id})

        return self.relation_model.objects.filter(**_filter).exists()

    def rm_relation(self, relation_type, target):

        """ Remove relation with given type with target as receiving
        end."""

        self.relation_model.objects.filter(
            src_content_type=self.ct_class,
            src_object_id=self.id,
            relation_type=relation_type,
            tgt_content_type=target.ct_class,
            tgt_object_id=target.id).delete()

    def rm_all_relations(self, inverse=True):

        """ Remove all relations. If inverse is true=ish, also remove
        relations where self is the target. """

    def save_relations(self):

        """ Save all relations set for the current object """

        # if this instance has a '_relation_updater' attribute, then
        # iterate over them, and call the update method
        if hasattr(self, '_relation_updater'):
            for updater in self._relation_updater:
                updater.update()
            # clean up, so another save won't mess up
            delattr(self, '_relation_updater')


class RelatableModel(models.Model, RelatableMixin):

    """ A model with the relatable stuff builtin """

    class Meta:

        """ Make the base model abstract """

        abstract = True

    def save(self):
        """ save the model + the relations """

        super(RelatableModel, self).save()
        self.save_relations()



from datetime import datetime
from django.core.management.base import BaseCommand
from django.db.models import Q
from djinn_contenttypes.models.publishable import PublishableContent
from djinn_contenttypes.registry import CTRegistry
from django.utils import translation


class Command(BaseCommand):

    help = """Publish contenttypes if need be"""

    def handle(self, *args, **options):

        """Check contentitems that just have reached published state
        due to publish timestamp. We just call 'save' on the instance,
        and let the signal handlers take care of the rest. """

        now = datetime.now()
        translation.activate("nl_NL")

        for ctype in CTRegistry.list_types():

            model = CTRegistry.get_attr(ctype, "class")

            if issubclass(model, PublishableContent):

                for instance in model.objects.filter(publish_notified=False,
                                                     publish_from__lt=now).\
                        filter(Q(publish_to__isnull=True) |
                        Q(publish_to__gt=now)):

                    # print "publishing %s-%d" % (instance.ct_name, instance.id)
                    instance.save()



class CTRegistry(object):

    """ Global registry for content types

    The registry can take any attrivutes per content type, but the following
    attributes have active use so far:

      class            Actual class for ct
      app              App id
      label            Front end name of CT
      global_add       Whether the ct should show in the add menu
      global_filter    Whether the ct should show in the search filters
      add_permission   Permission to determine whether a user can add the CT
      view_permission  Permission to determine whether a user can use the CT in
                       search filters.
      del_permission   Permission to determine whether a user can delete the CT
      edit_permission  Permission to check whether a user can edit the CT
      name_plural      well...
      filter_label     Label to show in filter. If empty, not shown at all.
      group_add        Whether the CT can be added to group context

    """

    content_types = {}

    @staticmethod
    def register(name, register_dict):

        CTRegistry.content_types[name] = register_dict

        # Also register with full name e.g. app.ctname
        CTRegistry.content_types["%s.%s" % (register_dict['app'], name)] = \
            register_dict

    @staticmethod
    def get(name):

        """
        Fetch all details for contenttype with name as a dict.
        """

        return CTRegistry.content_types.get(name, {})

    @staticmethod
    def get_attr(name, attr, default=None):

        """ Fetch given attr. Return default if not set"""

        return CTRegistry.content_types.get(name, {}).get(attr, default)

    @staticmethod
    def list_types(excludes=[]):

        """ Only list the 'full' keys: those with the app id and model id """

        return filter(lambda x: x not in excludes and "." in x,
                      CTRegistry.content_types.keys())



from datetime import datetime
from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
import django.dispatch
from haystack import signal_processor
from djinn_contenttypes.models.publishable import PublishableContent
from djinn_contenttypes.models.base import BaseContent
from djinn_contenttypes.models.history import (
    CREATED, CHANGED, PUBLISHED, UNPUBLISHED, History)
from djinn_core.utils import implements
from djinn_workflow.signals import state_change


unpublish = django.dispatch.Signal(providing_args=["instance"])
publish = django.dispatch.Signal(providing_args=["instance"])
created = django.dispatch.Signal(providing_args=["instance"])
changed = django.dispatch.Signal(providing_args=["instance"])


@receiver(unpublish)
def basecontent_unpublish(sender, instance, **kwargs):

    """ If publishable content has the remove_after_publish_to flag set,
    and the publish_to date is in the past, remove the instance. """

    if implements(instance, PublishableContent):

        now = datetime.now()

        if instance.remove_after_publish_to:

            if instance.publish_to and instance.publish_to < now:

                instance.delete()


@receiver(post_save)
def basecontent_post_save(sender, instance, **kwargs):

    if kwargs.get('created', False) and implements(instance, BaseContent):
        History.objects.log(instance, CREATED, user=instance.creator)
        created.send(sender, instance=instance)
    elif implements(instance, BaseContent):
        History.objects.log(instance, CHANGED, user=instance.changed_by)
        changed.send(sender, instance=instance)


@receiver(post_delete)
def basecontent_post_delete(sender, instance, **kwargs):

    if implements(instance, BaseContent):
        History.objects.delete_instance_log(instance)


@receiver(state_change)
def publishable_state_change(sender, instance, **kwargs):

    """ After a state change, always call reindex given that a change of
    state usually means a change in permissions"""

    publishable_post_save(sender, instance, **kwargs)
    signal_processor.handle_save(sender, instance)


@receiver(post_save)
def publishable_post_save(sender, instance, **kwargs):

    """Publishable post save hook. If the content is new and 'is_public'
    is true:
    * if  there is no history of publishing, send publish with first_edition
      flag.
    * if the content does have a publishing history, omit this, but signal
      published.
    """

    if implements(instance, PublishableContent):

        if instance.is_public:

            changed = False

            # Have we not been published before?
            #

            if not History.objects.has_been(instance, PUBLISHED, UNPUBLISHED):

                publish.send(sender, instance=instance, first_edition=True)
                changed = True

            # Yes we have. But maybe currently we're unpublished?
            #
            elif History.objects.get_last(
                    instance, PUBLISHED, UNPUBLISHED,
                    as_flag=True) == UNPUBLISHED:

                publish.send(sender, instance=instance)
                changed = True

            if changed:
                History.objects.log(instance, PUBLISHED,
                                    user=instance.changed_by)
                instance.__class__.objects.filter(pk=instance.pk).update(
                    publish_notified=True, unpublish_notified=False)

        else:
            # We're are not public. So if the last state was
            # 'published', actively unpublish.
            #
            last_state = History.objects.get_last(
                instance, PUBLISHED, UNPUBLISHED,
                as_flag=True)

            if last_state == PUBLISHED:

                unpublish.send(sender, instance=instance)
                instance.__class__.objects.filter(pk=instance.pk).update(
                    unpublish_notified=True)

                # The instance may not be there anymore...
                #
                if not instance.is_deleted:
                    History.objects.log(instance, UNPUBLISHED,
                                        user=instance.changed_by)



from datetime import datetime, timedelta
from django.test.testcases import TestCase
from django.db import models
from django.contrib.auth import get_user_model
from djinn_contenttypes.models.signal_processors import unpublish, publish


class PublishableTest(TestCase):

    def setUp(self):

        news_model = models.get_model("djinn_news", "News")
        user_model = get_user_model()

        self.user = user_model.objects.create(username="bobdobalina")
        self.content = news_model.objects.create(
            changed_by=self.user,
            title="test news",
            creator=self.user)

    def test_is_public(self):

        tomorrow = datetime.now() + timedelta(days=1)
        yesterday = datetime.now() - timedelta(days=1)

        # If nothing is said about publish_from or publish_to, the content is
        # public.
        #
        self.assertTrue(self.content.is_public)

        self.content.publish_from = tomorrow

        self.assertFalse(self.content.is_public)

        self.content.publish_from = yesterday

        self.assertTrue(self.content.is_public)

        self.content.publish_to = tomorrow

        self.assertTrue(self.content.is_public)

        self.content.publish_to = yesterday

        self.assertFalse(self.content.is_public)

    def test_is_published(self):

        tomorrow = datetime.now() + timedelta(days=1)
        yesterday = datetime.now() - timedelta(days=1)

        # If nothing is said about publish_from or publish_to, the content is
        # public.
        #
        self.assertTrue(self.content.is_published)

        self.content.publish_from = tomorrow

        self.assertFalse(self.content.is_published)

        self.content.publish_from = yesterday

        self.assertTrue(self.content.is_published)

        self.content.publish_to = tomorrow

        self.assertTrue(self.content.is_published)

        self.content.publish_to = yesterday

        self.assertFalse(self.content.is_published)

    def test_unpublish(self):

        tomorrow = datetime.now() + timedelta(days=1)

        self.assertTrue(self.content.is_public)

        def unpublish_callback(sender, instance, **kwargs):

            instance.publish_notified = False

        unpublish.connect(unpublish_callback)

        self.content.publish_notified = True
        self.content.publish_from = tomorrow
        self.content.save()

        self.assertFalse(self.content.publish_notified)

    def test_publish(self):

        tomorrow = datetime.now() + timedelta(days=1)
        yesterday = datetime.now() + timedelta(days=-1)
        signalled = []

        def publish_callback(sender, instance, **kwargs):

            signalled.append("published")

        publish.connect(publish_callback)

        self.assertTrue(self.content.is_published)

        self.content.publish_from = tomorrow
        self.content.save()

        self.assertFalse(self.content.is_published)

        self.content.publish_from = datetime.now()
        self.content.save()

        self.assertTrue(self.content.is_published)
        self.assertTrue(len(signalled) == 1 and signalled[0] == "published")

        self.content.publish_from = yesterday
        self.content.publish_to = yesterday
        self.content.save()

        self.assertFalse(self.content.is_published)

        self.content.publish_to = None
        self.content.save()

        self.assertTrue(self.content.is_published)
        self.assertTrue(len(signalled) == 2 and signalled[1] == "published")

    def test_remove_after_unpublish(self):

        yesterday = datetime.now() + timedelta(days=-1)
        tomorrow = datetime.now() + timedelta(days=1)

        self.content.publish_to = yesterday

        self.content.save()

        self.assertFalse(self.content.is_deleted)

        self.content.publish_to = tomorrow

        self.content.save()

        self.assertFalse(self.content.is_deleted)

        self.content.publish_to = yesterday
        self.content.remove_after_publish_to = True

        self.content.save()

        self.assertTrue(self.content.is_deleted)



from datetime import datetime, timedelta
from django.test.testcases import TestCase
from django.db import models
from django.contrib.auth import get_user_model
from djinn_contenttypes.models.signal_processors import (
    unpublish, publish, created, changed)


class BaseContentTest(TestCase):

    def setUp(self):

        news_model = models.get_model("djinn_news", "News")
        user_model = get_user_model()

        self.user = user_model.objects.create(username="bobdobalina")
        self.content = news_model.objects.create(
            changed_by=self.user,
            title="test news",
            creator=self.user)

    def test_deleted(self):

        self.assertFalse(self.content.is_deleted)

        self.content.delete()

        self.assertTrue(self.content.is_deleted)

    def test_viewers(self):

        self.assertTrue("group_users" in self.content.viewers)

        self.content.set_owner(self.user)

        self.assertTrue("user_bobdobalina" in self.content.viewers)

        self.assertEquals(2, len(self.content.viewers))

        tomorrow = datetime.now() + timedelta(days=1)

        self.content.publish_from = tomorrow
        self.content.save()

        self.assertFalse("group_users" in self.content.viewers)

        self.assertTrue("user_bobdobalina" in self.content.viewers)

        self.assertEquals(1, len(self.content.viewers))

    def testlifecycle(self):

        callbacks = []

        def unpublish_callback(sender, instance, **kwargs):

            callbacks.append("unpublish")

        def publish_callback(sender, instance, **kwargs):

            if kwargs.get("first_edition"):
                callbacks.append("publish_first")
            else:
                callbacks.append("publish")

        def created_callback(sender, instance, **kwargs):

            callbacks.append("created")

        def changed_callback(sender, instance, **kwargs):

            callbacks.append("changed")

        unpublish.connect(unpublish_callback)
        publish.connect(publish_callback)
        created.connect(created_callback)
        changed.connect(changed_callback)

        tomorrow = datetime.now() + timedelta(days=1)

        news_model = models.get_model("djinn_news", "News")

        content = news_model.objects.create(
            changed_by=self.user,
            title="test news",
            creator=self.user)

        self.assertTrue("created" == callbacks[0])
        self.assertTrue("publish_first" == callbacks[1])
        self.assertEquals(2, len(callbacks))

        content.publish_from = tomorrow
        content.save()

        self.assertTrue("changed" == callbacks[-2])
        self.assertTrue("unpublish" == callbacks[-1])
        self.assertEquals(4, len(callbacks))

        content.publish_from = None
        content.save()

        self.assertTrue("changed" == callbacks[-2])
        self.assertTrue("publish" == callbacks[-1])
        self.assertEquals(6, len(callbacks))



from django.template import Library
from django.utils.translation import ugettext as _
from django.conf import settings
from django.utils import dateformat
from djinn_workflow.utils import get_state


register = Library()


@register.simple_tag
def scheduled_line(obj):

    """ Return the object's 'scheduled' publishing """

    line = ""

    if obj.publish_from and obj.publish_to:

        line = _("From %(publish_from)s till %(publish_to)s") % {
            'publish_from': dateformat.format(obj.publish_from,
                                              settings.DATETIME_FORMAT),
            'publish_to': dateformat.format(obj.publish_to,
                                            settings.DATETIME_FORMAT)}

    elif obj.publish_from:

        line = _("From %(publish_from)s") % {
            'publish_from': dateformat.format(obj.publish_from,
                                              settings.DATETIME_FORMAT)}

    elif obj.publish_to:

        line = _("Till %(publish_to)s") % {
            'publish_to': dateformat.format(obj.publish_to,
                                            settings.DATETIME_FORMAT)}

    return line


@register.simple_tag
def pub_classes(obj):

    """ publishing related CSS classes """

    classes = []

    # if not implements(obj, PublishableContent):
    if not hasattr(obj, "is_published"):
        return ""

    if not obj.is_published:
        classes.append("unpublished")
    else:
        classes.append("published")

    if hasattr(obj, 'is_scheduled') and obj.is_scheduled:
        classes.append("scheduled")

    try:
        classes.append(get_state(obj).name)
    except:
        pass

    return " ".join(classes)



from django.db import models
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic


CREATED = 1
CHANGED = 2
PUBLISHED = 3
UNPUBLISHED = 4


STATUS_MAP = ["", "created", "changed", "published", "unpublished"]


class HistoryManager(models.Manager):

    def log(self, instance, status_flag, user=None, change_message=''):

        entry = self.model(
            user=user,
            object_id=instance.id,
            object_ct=ContentType.objects.get_for_model(instance),
            status_flag=status_flag,
            change_message=change_message)
        entry.save()

    def get_last(self, instance, *flags, **kwargs):

        _filter = {
            'object_id': instance.id,
            'object_ct': ContentType.objects.get_for_model(instance)}

        if flags:
            _filter['status_flag__in'] = flags

        # NB: ordering is already on -date, so get _first_ here!
        #
        try:
            if kwargs.get("as_flag"):
                return self.filter(**_filter).first().status_flag
            else:
                return self.filter(**_filter).first()
        except:
            return None

    def has_been(self, instance, *flags):

        _filter = {
            'object_id': instance.id,
            'object_ct': ContentType.objects.get_for_model(instance)}

        if flags:
            _filter['status_flag__in'] = flags

        return self.filter(**_filter).exists()

    def delete_instance_log(self, instance):

        _filter = {
            'object_id': instance.id,
            'object_ct': ContentType.objects.get_for_model(instance)}

        self.filter(**_filter).delete()


class History(models.Model):

    """ status history of content """

    date = models.DateTimeField(auto_now_add=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True)
    object_ct = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('object_ct', 'object_id')
    status_flag = models.PositiveSmallIntegerField()
    change_message = models.TextField(blank=True)

    objects = HistoryManager()

    def __unicode__(self):

        try:
            status = STATUS_MAP[self.status_flag]
        except:
            status = self.status_flag

        try:
            return u"'%s' %s" % (self.content_object, status)
        except:
            # may give unicode error
            return u"History(id=%d) %s" % (self.id, status)

    class Meta:

        app_label = "djinn_contenttypes"
        ordering = ["-date"]



from datetime import datetime
from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.translation import ugettext_lazy as _


class Highlight(models.Model):

    """Hightlight given content. This can be used to set an additional
    period of highlighting, for instance to publish on the home page
    of the intranet

    """

    date_from = models.DateTimeField(
        _('Highlight from'), null=True,
        blank=True)

    date_to = models.DateTimeField(
        _('Highlight to'), null=True,
        blank=True)

    object_ct = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('object_ct', 'object_id')

    @property
    def human_date(self):

        delta = datetime.now() - self.date_from

        if delta.days == 0:
            return self.date_from.strftime('%H:%M')
        else:
            return self.date_from.strftime('%d-%m')

    def __unicode__(self):

        return "%s highlight" % self.content_object

    class Meta:

        app_label = "djinn_contenttypes"
        unique_together = ("object_ct", "object_id")



from django.conf import settings

DEFAULT_DATETIME_INPUT_FORMAT = getattr(
    settings, 'DEFAULT_DATETIME_INPUT_FORMAT', '%d/%m/%Y %H:%M')

DEFAULT_DATE_INPUT_FORMAT = getattr(
    settings, 'DEFAULT_DATE_INPUT_FORMAT', '%d/%m/%Y')


from pgauth.base import Role
from pgauth.settings import VIEWER_ROLE_ID, EDITOR_ROLE_ID
from djinn_contenttypes.utils import get_object_by_ctype_id


class SharingMixin(object):

    def add_share(self, ctype, cid, mode):

        """ Add share to given ct, with mode """

        # Import runtime to prevent dependency hell.
        # TODO needs to be resolved obviously...
        #
        from pgevents.settings import SHARE_CONTENT
        from pgevents.events import Events

        if mode == 'viewer':
            role = Role.objects.get(name=VIEWER_ROLE_ID)
            mode = "bekijken"
        elif mode == 'editor':
            role = Role.objects.get(name=EDITOR_ROLE_ID)
            mode = "bewerken"

        tgt = get_object_by_ctype_id(ctype, cid)

        if getattr(tgt, "user", None):
            self.add_local_role(role, tgt.user)
            Events.send(SHARE_CONTENT,
                        self.get_owner(),
                        users=[tgt.user],
                        content=self, mode=mode)
        elif getattr(tgt, "usergroup", None):
            self.add_local_role(role, tgt.usergroup)

            Events.send(SHARE_CONTENT,
                        self.get_owner(),
                        users=tgt.usergroup.members.all(),
                        content=self, mode=mode)
        else:
            raise

    def rm_share(self, ctype, cid, mode):

        """ Remove share to given ct, with mode """

        if mode == 'viewer':
            role = Role.objects.get(name=VIEWER_ROLE_ID)
            mode = "bekijken"
        elif mode == 'editor':
            role = Role.objects.get(name=EDITOR_ROLE_ID)
            mode = "bewerken"

        tgt = get_object_by_ctype_id(ctype, cid)

        if getattr(tgt, "user", None):
            self.rm_local_role(role, tgt.user)
        elif getattr(tgt, "usergroup", None):
            self.rm_local_role(role, tgt.usergroup)
        else:
            raise

    @property
    def shares(self):
        return self.get_local_roles(
            role_filter=[EDITOR_ROLE_ID, VIEWER_ROLE_ID])

    @property
    def user_shares(self):
        return self.get_local_roles(
            role_filter=[EDITOR_ROLE_ID, VIEWER_ROLE_ID]).filter(
            user__isnull=False)

    @property
    def group_shares(self):
        return self.get_local_roles(
            role_filter=[EDITOR_ROLE_ID, VIEWER_ROLE_ID]).filter(
            usergroup__isnull=False)



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'django',
    'requests',
    'djinn_core',
    'django-bootstrap-form',
    'djinn_forms>=1.2.3',
    'djinn_workflow'
    ]

setup(name='djinn_contenttypes',
      version="1.4.8",
      description='Djinn Intranet Contenttypes framework',
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Framework :: Django",
          "Intended Audience :: Developers",
          "License :: Freely Distributable",
          "Programming Language :: Python",
          "Topic :: Internet :: WWW/HTTP :: Site Management",
          "Topic :: Software Development :: Libraries :: "
          "Application Frameworks"
      ],
      author='PythonUnited',
      author_email='info@pythonunited.com',
      license='beer-ware',
      url='https://github.com/PythonUnited/djinn-contenttypes',
      keywords='Djinn Contenttypes',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      tests_require=requires,
      test_suite="djinn-contenttypes",
      entry_points="""\
      [djinn.app]
      js=djinn_contenttypes:get_js
      css=djinn_contenttypes:get_css
      urls=djinn_contenttypes:get_urls
      """
      )



from django.test.testcases import TestCase
from django.db import models
from django.contrib.auth import get_user_model
from djinn_contenttypes.models.history import (
    History, CHANGED, CREATED, PUBLISHED, UNPUBLISHED)


class HistoryTest(TestCase):

    def setUp(self):

        news_model = models.get_model("djinn_news", "News")
        user_model = get_user_model()

        self.user = user_model.objects.create(username="bobdobalina")
        self.content = news_model.objects.create(
            changed_by=self.user,
            title="test news",
            creator=self.user)

    def test_log(self):

        self.assertTrue(CREATED,
                        History.objects.get_last(self.content).status_flag)

        History.objects.log(self.content, CHANGED)

        self.assertTrue(CHANGED,
                        History.objects.get_last(self.content).status_flag)

    def test_get_last(self):

        """ Some more elaborate testing, to find out about the published/
        unpublished status of content """

        History.objects.log(self.content, PUBLISHED)

        self.assertTrue(PUBLISHED,
                        History.objects.get_last(
                            self.content,
                            PUBLISHED, UNPUBLISHED).status_flag)

        History.objects.log(self.content, CHANGED)

        self.assertTrue(PUBLISHED,
                        History.objects.get_last(
                            self.content,
                            PUBLISHED, UNPUBLISHED).status_flag)

        History.objects.log(self.content, UNPUBLISHED)

        self.assertTrue(UNPUBLISHED,
                        History.objects.get_last(
                            self.content,
                            PUBLISHED, UNPUBLISHED).status_flag)

        History.objects.log(self.content, PUBLISHED)

        self.assertTrue(PUBLISHED,
                        History.objects.get_last(
                            self.content,
                            PUBLISHED, UNPUBLISHED).status_flag)

    def test_auto_cleanup(self):

        History.objects.log(self.content, PUBLISHED)

        self.content.delete()

        self.assertEquals(None, History.objects.get_last(self.content))
